----------------------------------------------------------------------------------
-- controller.vhd
--
-- Copyright (C) 2006 Michael Poppitz
-- 
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc.,
-- 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
--
----------------------------------------------------------------------------------
--
-- Details: http://www.sump.org/projects/analyzer/
--
-- Controls the capturing & readback operation.
-- 
-- If no other operation has been activated, the controller samples data
-- into the memory. When the run signal is received, it continues to do so
-- for fwd * 4 samples and then sends bwd * 4 samples  to the transmitter.
-- This allows to capture data from before the trigger match which is a nice 
-- feature.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity controller is
  port (
    clock : in std_logic;
    reset : in std_logic;

    input      : in std_logic_vector (31 downto 0);  --the sampled data
    inputReady : in std_logic;                       --flag - new data sampled

    run        : in  std_logic;         --from trigger
    wrfwd      : in  std_logic;         --from decoder, valid for one cycle
    wrbwd      : in  std_logic;         --from decoder, valid for one cycle
    cmd        : in  std_logic_vector (31 downto 0);  --cmd(31 downto 8) - holds the control-data
    arm        : in  std_logic;         --from the decoder, creates bwd-samples
    armtrigger : out std_logic;  --arms the trigger, pulse, 1 clock cycle long.

    memoryOut   : out std_logic_vector (31 downto 0);  --to dram
    memoryWrite : out std_logic;                       --to dram

    sample_finish_IRQxSO : out std_logic;  --interrupt-output-flag for the AVR. (finished sampling)
    sample_clrflagxSI    : in  std_logic   --input to clear the interrupt flag
    );
end controller;

architecture Behavioral of controller is

  type CONTROLLER_STATES is (SAMPLE, FWD_SAMPLE, BWD_SAMPLE, IRQWAIT, IDLE);  --READ_WAIT_MEM, READ_WAIT_TX); --state machine

  -- counter, counts how long to sample, big enough for 128MiB (25 bit are sufficient)
  signal fwd, bwd                 : std_logic_vector (24 downto 0);
  -- sample counters, big enough for 128MiB (25 bit)
  signal nstate, state            : CONTROLLER_STATES := IDLE;
  signal samplexDN, samplexDP     : std_logic_vector (31 downto 0);
  signal memWritexDN, memWritexDP : std_logic;
  signal armTriggerxSN, armTriggerxSP : std_logic := '0';
  -- signals for DSP slice counter
  signal counter                  : std_logic_vector (24 downto 0);
  signal incCntxS                 : std_logic;
  signal clrCntxS                 : std_logic;
  signal dspPxD                   : std_logic_vector (47 downto 0);
  

begin
  -- insert a flip flop into the data path, we need a flip flop on the memoryWrite signal to improve timings, this is here to 
  -- keep the data aligned with the memoryWrite strobe
  memoryOut <= samplexDP;
  samplexDN <= input;  --always put the sampled data to the memory

  memoryWrite <= memWritexDP;
  armTrigger <= armTriggerxSP;

  -- synchronization and reset logic
  process(clock, reset)
  begin
	armTriggerxSP <= armTriggerxSN;
    if reset = '1' then
      state <= IDLE;                    --idle state is the wait state
      --   counter <= (others => '0');
    elsif rising_edge(clock) then
      state <= nstate;
      --   counter <= ncounter;
    end if;
  end process;

  -- FSM to control the controller action
  process(arm, bwd, counter, fwd, inputReady, run, sample_clrflagxSI, state)
  begin
    --default assignments:
    nstate               <= state;
    clrCntxS             <= '0';
    incCntxS             <= '0';
    --ncounter  <= counter;
    memWritexDN          <= '0';
    sample_finish_IRQxSO <= '0';
    armTriggerxSN        <= '0';

    case state is
      
      when IDLE =>         --idle-wait-state, do nothing, wait for "arm".
        if arm = '1' then               --if bwd-samples needed
          nstate <= BWD_SAMPLE;
        end if;
        --ncounter <= (others => '0');
        clrCntxS <= '1';

      when BWD_SAMPLE =>                --sample the bwd-samples.
        if counter = bwd then           --if all bwd-samples sampled
--        ncounter   <= (others => '0');
--          clrCntxS <= '1';
          nstate     <= SAMPLE;         --sample the forward data
          armTriggerxSN <= '1';     --arm the trigger now! (for one cycle high)
        else
--          if inputReady = '1' then      --data ready(from sampler)
--            ncounter <= counter + 1;
--          else
--            ncounter <= counter;
--          end if;
          incCntxS <= inputReady;
        end if;
        memWritexDN <= inputReady;
        
      when SAMPLE =>
        if run = '1' then  --trigger sets run to '1' ==> begin sampling and storing from here!
          nstate <= FWD_SAMPLE;
        end if;
--        ncounter    <= (others => '0');  --reset counter
        clrCntxS    <= '1';
        memWritexDN <= inputReady;  --this goes high, as soon as new data are sampled ==> write

        -- keep sampling for "fwd" samples after run condition
      when FWD_SAMPLE =>
--        ncounter    <= counter;         -- default: do nothing
        memWritexDN <= inputReady;      --write to memory, if new data ready.
        incCntxS    <= inputReady;
        if inputReady = '1' then
          if counter = fwd then         --sampled enough
            --          ncounter    <= (others => '0');
            memWritexDN <= '0';
            nstate      <= IRQWAIT;     --goto next state
--          else
            --new data ready
            --        ncounter <= counter + 1;    --increase the counter
          end if;
        end if;

        --all data has been sampled ==> wait for the AVR to confirm the IRQ.
      when IRQWAIT =>
        sample_finish_IRQxSO <= '1';    --set IRQ-flag
        if sample_clrflagxSI = '1' then  --if the AVR clears the flag 
          nstate <= IDLE;               --go to beginning
--          ncounter <= (others => '0');
        else
          nstate <= state;  --else - wait here until AVR clears the flag.
        end if;
        
    end case;
  end process;

  -- set speed and size registers if indicated
  process(clock, reset)
  begin
    if reset = '1' then
      fwd         <= (others => '0');
      bwd         <= (others => '0');
      memWritexDP <= '0';
      samplexDP   <= (others => '0');
    elsif rising_edge(clock) then
      memWritexDP <= memWritexDN;
      samplexDP   <= samplexDN;
      if wrfwd = '1' then               --adjust fwd data
        fwd <= cmd(24 downto 0);
      elsif wrbwd = '1' then
        bwd <= cmd(24 downto 0);
      end if;
    end if;
  end process;

  -- DSP slice used as fast counter

  counter <= dspPxD(24 downto 0);

  DSP48A1_cntr_inst : DSP48A1
    generic map (
      A0REG       => 0,  -- First stage A input pipeline register (0/1)
      A1REG       => 1,  -- Second stage A input pipeline register (0/1)
      B0REG       => 0,  -- First stage B input pipeline register (0/1)
      B1REG       => 1,  -- Second stage B input pipeline register (0/1)
      CARRYINREG  => 1,  -- CARRYIN input pipeline register (0/1)
      CARRYINSEL  => "OPMODE5",  -- Specify carry-in source, "CARRYIN" or "OPMODE5" 
      CARRYOUTREG => 1,  -- CARRYOUT output pipeline register (0/1)
      CREG        => 1,                 -- C input pipeline register (0/1)
      DREG        => 1,  -- D pre-adder input pipeline register (0/1)
      MREG        => 1,                 -- M pipeline register (0/1)
      OPMODEREG   => 0,  -- Enable=1/disable=0 OPMODE input pipeline registers
      PREG        => 1,                 -- P output pipeline register (0/1)
      RSTTYPE     => "SYNC"     -- Specify reset type, "SYNC" or "ASYNC" 
      )
    port map (
      -- Cascade Ports: 18-bit (each) Ports to cascade from one DSP48 to another
      BCOUT     => open,                -- 18-bit B port cascade output
      PCOUT     => open,  -- 48-bit P cascade output (if used, connect to PCIN of another DSP48A1)
      -- Data Ports: 1-bit (each) Data input and output ports
      CARRYOUT  => open,  -- 1-bit carry output (if used, connect to CARRYIN pin of another DSP48A1)
      CARRYOUTF => open,                -- 1-bit fabric carry output
      M         => open,                -- 36-bit fabric multiplier data output
      P         => dspPxD,              -- 48-bit data output
      -- Cascade Ports: 48-bit (each) Ports to cascade from one DSP48 to another
      PCIN      => x"000000000000",  -- 48-bit P cascade input (if used, connect to PCOUT of another DSP48A1)
      -- Control Input Ports: 1-bit (each) Clocking and operation mode
      CLK       => clock,               -- 1-bit clock input
      OPMODE    => "00001110",  -- configure post adder to accumulate data on C port
      -- Data Ports: 18-bit (each) Data input and output ports
      A         => "000000000000000000",  -- 18-bit A data input
      B         => "000000000000000000",  -- 18-bit B data input (connected to fabric or BCOUT of adjacent DSP48A1)
      C         => x"000000000001",     -- 48-bit C data input
      CARRYIN   => '0',  -- 1-bit carry input signal (if used, connect to CARRYOUT pin of another
      -- DSP48A1)

      D          => "000000000000000000",  -- 18-bit B pre-adder data input
      -- Reset/Clock Enable Input Ports: 1-bit (each) Reset and enable input ports
      CEA        => '0',  -- 1-bit active high clock enable input for A registers
      CEB        => '0',  -- 1-bit active high clock enable input for B registers
      CEC        => '1',  -- 1-bit active high clock enable input for C
-- registers. We apply a fixed value to C so we simply enable this constantly
      CECARRYIN  => '0',  -- 1-bit active high clock enable input for CARRYIN registers
      CED        => '0',  -- 1-bit active high clock enable input for D registers
      CEM        => '0',  -- 1-bit active high clock enable input for multiplier registers
      CEOPMODE   => '1',  -- 1-bit active high clock enable input for OPMODE registers
      CEP        => incCntxS,  -- 1-bit active high clock enable input for P registers
      RSTA       => '0',  -- 1-bit reset input for A pipeline registers
      RSTB       => '0',  -- 1-bit reset input for B pipeline registers
      RSTC       => '0',  -- 1-bit reset input for C pipeline registers
      RSTCARRYIN => '0',  -- 1-bit reset input for CARRYIN pipeline registers
      RSTD       => '0',  -- 1-bit reset input for D pipeline registers
      RSTM       => '0',  -- 1-bit reset input for M pipeline registers
      RSTOPMODE  => '0',  -- 1-bit reset input for OPMODE pipeline registers
      RSTP       => clrCntxS   -- 1-bit reset input for P pipeline registers
      );


end Behavioral;
