--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This unit decodes received from the cpu into individual write strobes. 
--*
--***************************************************************************************************************
----------------------------------------------------------------------------------
-- decoder.vhd
--
-- Copyright (C) 2006-2011 Michael Poppitz
-- 
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc.,
-- 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
--
----------------------------------------------------------------------------------
--
-- Details: http://www.sump.org/projects/analyzer/
--
-- Takes the opcode from the command received by the receiver and decodes it.
-- The decoded command will be executed for one cycle.
--
-- The receiver keeps the cmd output active long enough so all the
-- data is still available on its cmd output when the command has
-- been decoded and sent out to other modules with the next
-- clock cycle. (Maybe this paragraph should go in receiver.vhd?)
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity decoder is
  port (
    ClkxCI             : in  std_logic;
    opcodexDI          : in  std_logic_vector (7 downto 0);  -- specifies the command
    dataxDI            : in  std_logic_vector(31 downto 0);  -- additional data for the command
    executexSI         : in  std_logic;  -- indicates that opcode is valid
    -- write strobe outputs
    wrTrigUnitSelxSO   : out std_logic;
    wrTrigMaskxSO      : out std_logic;
    wrTrigValxSO       : out std_logic;
    wrTrigFlagsxSO     : out std_logic;
    wrTrigDelayxSO     : out std_logic;
    wrSpeedxSO         : out std_logic;
    wrFwdxSO           : out std_logic;
    wrBwdxSO           : out std_logic;
    -- flag outputs
    flagDemuxxSO       : out std_logic;
    flagFilterxSO      : out std_logic;
    flagExtClkxSO      : out std_logic;
    flagFallingEdgexSO : out std_logic;
    -- special commands
    runxSO             : out std_logic;
    resetCorexRO       : out std_logic
    );
end decoder;

architecture Behavioral of decoder is


  signal runxSN, runxSP                     : std_logic := '0';
  signal resetCorexRN, resetCorexRP         : std_logic := '0';
  signal wrSpeedxSN, wrSpeedxSP             : std_logic := '0';
  signal wrFwdxSN, wrFwdxSP                 : std_logic := '0';
  signal wrBwdxSN, wrBwdxSP                 : std_logic := '0';
  signal wrTrigMaskxSN, wrTrigMaskxSP       : std_logic := '0';
  signal wrTrigValxSN, wrTrigValxSP         : std_logic := '0';
  signal wrTrigFlagsxSN, wrTrigFlagsxSP     : std_logic := '0';
  signal wrTrigDelayxSN, wrTrigDelayxSP     : std_logic := '0';
  signal wrTrigUnitSelxSN, wrTrigUnitSelxSP : std_logic := '0';
  signal wrFlagsxSN, wrFlagsxSP             : std_logic := '0';

  signal flagDemuxxSN, flagDemuxxSP             : std_logic := '0';
  signal flagFilterxSN, flagFilterxSP           : std_logic := '0';
  signal flagExtClkxSN, flagExtClkxSP           : std_logic := '0';
  signal flagFallingEdgexSN, flagFallingEdgexSP : std_logic := '0';
  
begin

  process (executexSI, opcodexDI, runxSP)
  begin  -- process
    -- default: do nothing
    resetCorexRN     <= '0';
    runxSN           <= runxSP;
    wrSpeedxSN       <= '0';
    wrFlagsxSN       <= '0';
    wrFwdxSN         <= '0';
    wrBwdxSN         <= '0';
    wrTrigMaskxSN    <= '0';
    wrTrigValxSN     <= '0';
    wrTrigFlagsxSN   <= '0';
    wrTrigDelayxSN   <= '0';
    wrTrigUnitSelxSN <= '0';

    if executexSI = '1' then
      case opcodexDI is
        when x"00" => resetCorexRN <= '1';
                      runxSN <= '0';
        when x"01" => runxSN           <= '1';
        when x"80" => wrSpeedxSN       <= '1';
        when x"82" => wrFlagsxSN       <= '1';
        when x"83" => wrFwdxSN         <= '1';  --the data in the cmd determines the number of fwd-samples
        when x"84" => wrBwdxSN         <= '1';  --the data in the cmd determines the number of bwd-samples
        when x"C0" => wrTrigMaskxSN    <= '1';
        when x"C1" => wrTrigValxSN     <= '1';
        when x"C2" => wrTrigFlagsxSN   <= '1';
        when x"C3" => wrTrigDelayxSN   <= '1';
        when x"C4" => wrTrigUnitSelxSN <= '1';
        when others =>
      end case;
    end if;
  end process;

  process (dataxDI, flagDemuxxSP, flagExtClkxSP, flagFallingEdgexSP,
           flagFilterxSP, resetCorexRP, wrFlagsxSP)
  begin  -- process

-- the flags stay at their previous value
    flagDemuxxSN       <= flagDemuxxSP;
    flagFilterxSN      <= flagFilterxSP;
    flagExtClkxSN      <= flagExtClkxSP;
    flagFallingEdgexSN <= flagFallingEdgexSP;

    if resetCorexRP = '1' then
      flagDemuxxSN       <= '0';
      flagFilterxSN      <= '0';
      flagExtClkxSN      <= '0';
      flagFallingEdgexSN <= '0';
    end if;

    if wrFlagsxSP = '1' then
      flagDemuxxSN       <= dataxDI(0);
      flagFilterxSN      <= dataxDI(1);
      flagExtClkxSN      <= dataxDI(6);
      flagFallingEdgexSN <= dataxDI(7);
    end if;
  end process;

  -- Note we have no reset signal here as there are no register that need to
  -- be reseted. It might make sens so reset the flags on poweron, however this is
  -- done by software using 'reset' command anyway so we save some fpga ressoureces
  regs : process (ClkxCI)
  begin  -- process regs

    if ClkxCI'event and ClkxCI = '1' then  -- rising clock edge
      resetCorexRP       <= resetCorexRN;
      runxSP             <= runxSN;
      wrSpeedxSP         <= wrSpeedxSN;
      wrFlagsxSP         <= wrFlagsxSN;
      wrFwdxSP           <= wrFwdxSN;
      wrBwdxSP           <= wrBwdxSN;
      wrTrigMaskxSP      <= wrTrigMaskxSN;
      wrTrigValxSP       <= wrTrigValxSN;
      wrTrigFlagsxSP     <= wrTrigFlagsxSN;
      wrTrigDelayxSP     <= wrTrigDelayxSN;
      wrTrigUnitSelxSP   <= wrTrigUnitSelxSN;
      flagDemuxxSP       <= flagDemuxxSN;
      flagFilterxSP      <= flagFilterxSN;
      flagExtClkxSP      <= flagExtClkxSN;
      flagFallingEdgexSP <= flagFallingEdgexSN;
    end if;
  end process regs;

  -- output assignments
  resetCorexRO       <= resetCorexRP;
  runxSO             <= runxSP;
  wrSpeedxSO         <= wrSpeedxSP;
  wrFwdxSO           <= wrFwdxSP;
  wrBwdxSO           <= wrBwdxSP;
  wrTrigMaskxSO      <= wrTrigMaskxSP;
  wrTrigValxSO       <= wrTrigValxSP;
  wrTrigFlagsxSO     <= wrTrigFlagsxSP;
  wrTrigDelayxSO     <= wrTrigDelayxSP;
  wrTrigUnitSelxSO   <= wrTrigUnitSelxSP;
  flagDemuxxSO       <= flagDemuxxSP;
  flagFilterxSO      <= flagFilterxSP;
  flagExtClkxSO      <= flagExtClkxSP;
  flagFallingEdgexSO <= flagFallingEdgexSP;
  
end Behavioral;
