--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This is the top level of the analyzer logic. It instanciates the core and it's whishbone interface.
--*
--***************************************************************************************************************
----------------------------------------------------------------------------------
-- la.vhd
--
-- Copyright (C) 2006 Michael Poppitz
-- 
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc.,
-- 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
--
----------------------------------------------------------------------------------
--
-- Details: http://www.sump.org/projects/analyzer/
--
-- Logic Analyzer top level module. It connects the core with the hardware
-- dependend IO modules and defines all inputs and outputs that represent
-- phyisical pins of the fpga.
--
-- It defines two constants FREQ and RATE. The first is the clock frequency 
-- used for receiver and transmitter for generating the proper baud rate.
-- The second defines the speed at which to operate the serial port.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.VComponents.all;

entity la is
  port(
    -- sampling clock domain (200MHz)
    reset                : in  std_logic;
    clock                : in  std_logic;
    clockB               : in  std_logic;  -- 180 degree phase shifted clock
    -- avr clock domain (40MHz)
    AVR_Clock            : in  std_logic;  -- AVR clock
    AVR_Reset            : in  std_logic;
    exClock              : in  std_logic;
    input                : in  std_logic_vector(31 downto 0);
    testOutput           : out std_logic;
    -- interface to sample memor
    memoryClkxCO         : out std_logic;
    memoryOut            : out std_logic_vector(31 downto 0);
    memoryWrite          : out std_logic;
    -- Wishbone Interface for configuration
    Wb_cycxSI            : in  std_logic;
    Wb_stbxSI            : in  std_logic;
    Wb_wexSI             : in  std_logic;
    Wb_ackxSO            : out std_logic;
    Wb_errxSO            : out std_logic;
    Wb_adrxDI            : in  std_logic_vector(3 downto 0);
    Wb_datxDI            : in  std_logic_vector(7 downto 0);
    Wb_datxDO            : out std_logic_vector(7 downto 0);
    --AVR-Interrupt:
    sample_finish_IRQxSO : out std_logic;  --interrupt-output-flag for the AVR. (finished sampling)
    sample_clrflagxSI    : in  std_logic   --input to clear the interrupt flag
    );
end la;

architecture Behavioral of la is

  component lfsr
    generic (
      CLK_DIV : integer;
      INITIAL : std_logic_vector (7 downto 0));
    port (rst  : in  std_logic;
          clk  : in  std_logic;
          dout : out std_logic_vector (7 downto 0));
  end component;

  component Wb_sump_interface
    port (
      ClkxCI      : in  std_logic;
      RstxRI      : in  std_logic;
      Wb_cycxSI   : in  std_logic;
      Wb_stbxSI   : in  std_logic;
      Wb_wexSI    : in  std_logic;
      Wb_ackxSO   : out std_logic;
      Wb_errxSO   : out std_logic;
      Wb_adrxDI   : in  std_logic_vector(3 downto 0);
      Wb_datxDI   : in  std_logic_vector(7 downto 0);
      Wb_datxDO   : out std_logic_vector(7 downto 0);
      Clk_sumpxCI : in  std_logic;
      Rst_sumpxRI : in  std_logic;
      channelsxDI : in  std_logic_vector(31 downto 0);
      cmd_dataxDO : out std_logic_vector(31 downto 0);
      opcodexDO   : out std_logic_vector(7 downto 0);
      executexSO  : out std_logic);
  end component;

  component core
    generic (
      CHANNELS : integer);
    port (
      ClkxCI               : in  std_logic;
      ClkxCBI              : in  std_logic;
      resetxRI             : in  std_logic;
      cmdDatxDI            : in  std_logic_vector (31 downto 0);
      opcodexDI            : in  std_logic_vector(7 downto 0);
      executexSI           : in  std_logic;
      channelsxDI          : in  std_logic_vector (31 downto 0);
      flagExtClkxSO        : out std_logic;
      sampleClkxSO         : out std_logic;
      memoryClkxCO         : out std_logic;
      memoryOut            : out std_logic_vector (31 downto 0);
      memoryWrite          : out std_logic;
      sample_finish_IRQxSO : out std_logic;
      sample_clrflagxSI    : in  std_logic);
  end component;

  component coreClkMgr
    port (
      IntClkxCI     : in  std_logic;
      IntClkxCBI    : in  std_logic;
      ExtClkxCI     : in  std_logic;
      DcmLockedxSO  : out std_logic;
      ResetxRI      : in  std_logic;
      FlagExtClkxSI : in  std_logic;
      SampleClkxCO  : out std_logic;
      SampleClkxCBO : out std_logic);
  end component;

  signal sampleClkxC, sampleClkxCB : std_logic;
  signal memoryClkxC               : std_logic;
  signal dcmLockedxS               : std_logic;

  signal cmd          : std_logic_vector (31 downto 0);
  signal opcode       : std_logic_vector(7 downto 0);
  --signal clock : std_logic;
  signal execute      : std_logic;
  signal lfsrOut      : std_logic_vector (7 downto 0);
  signal inputSignal  : std_logic_vector (31 downto 0);
  signal flagExtClkxS : std_logic;

  signal ready50 : std_logic;

  -- register for clock domain crossing of interrupt signals
  signal irqxSN, irqxSP   : std_logic;
  signal iackxSN, iackxSP : std_logic;

--This determines the Width of the LA ==> 32 or 64 Channels
-- NOTE: This does not work yet, don't set any other value than 32!!
  constant CHANNELS : integer := 32;
  
begin


  inputSignal <= input;
  testOutput  <= lfsrOut(0);

  inst_lfsr : lfsr
    generic map (
      CLK_DIV => 1,
      INITIAL => "00000001")
    port map (rst  => '0',              -- not meaningful
              clk  => AVR_clock,
              dout => lfsrOut
              );

  Inst_coreClkMgr : coreClkMgr
    port map (
      IntClkxCI     => clock,
      IntClkxCBI    => clockB,
      ExtClkxCI     => exClock,
      DcmLockedxSO  => dcmLockedxS,
      ResetxRI      => reset,
      FlagExtClkxSI => FlagExtClkxS,
      SampleClkxCO  => SampleClkxC,
      SampleClkxCBO => SampleClkxCB);


  Inst_Wb_sump_interface : Wb_sump_interface
    port map(
      --These are the connections for the Wishbone-Bus:(slave, 8 bit)
      ClkxCI      => AVR_clock,
      RstxRI      => AVR_Reset,
      Wb_cycxSI   => Wb_cycxSI,
      Wb_stbxSI   => Wb_stbxSI,
      Wb_wexSI    => Wb_wexSI,
      Wb_ackxSO   => Wb_ackxSO,
      Wb_errxSO   => Wb_errxSO,
      Wb_adrxDI   => Wb_adrxDI,
      Wb_datxDI   => Wb_datxDI,
      Wb_datxDO   => Wb_datxDO,
      --This is the interface to the core:
      Clk_sumpxCI => SampleClkxC,
      Rst_sumpxRI => reset,
      channelsxDI => inputSignal,
      cmd_dataxDO => cmd,
      opcodexDO   => opcode,
      executexSO  => execute
      );

  Inst_Core : core
    generic map (
      CHANNELS => CHANNELS)
    port map (
      ClkxCI               => SampleClkxC,
      ClkxCBI              => SampleClkxCB,
      ResetxRI             => reset,
      cmdDatxDI            => cmd,
      opcodexDI            => opcode,
      executexSI           => execute,
      channelsxDI          => inputSignal,
      flagExtClkxSO        => flagExtClkxS,
      sampleClkxSO         => ready50,
      memoryClkxCO         => memoryClkxC,
      memoryOut            => memoryOut,
      memoryWrite          => memoryWrite,
      sample_finish_IRQxSO => irqxSN,
      sample_clrflagxSI    => iackxSP);

  memoryClkxCO <= memoryClkxC;

  -- clock domain crossing of interrupt signals
  sample_finish_IRQxSO <= irqxSP;
  iackxSN              <= sample_clrflagxSI;
  process (AVR_Clock)
  begin
    if AVR_Clock'event and AVR_Clock = '1' then
      irqxSP <= irqxSN;
    end if;
  end process;

  process (clock)
  begin
    if clock'event and clock = '1' then
      iackxSP <= iackxSN;
    end if;
  end process;
  
end Behavioral;

