--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Namin convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This entity counts the number of samples once the input signal runxSI goes
--* high. RunxSI has to stay high for the entire counting process. Once the
--* counter has reached configured top value donexSO goes high and stays high until the
--* entity is resetted.
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity sampleCounter is
  generic (
    CNT_WIDTH : integer := 25);         -- counter width in bits, max 48 bits
  port (
    ClkxCI         : in  std_logic;
    RstxRI         : in  std_logic;
    -- sample data input
    samplexDI      : in  std_logic_vector(31 downto 0);
    sampleValidxSI : in  std_logic;
    -- control inputs
    runxSI         : in  std_logic;
    wrTopxSI       : in  std_logic;
    cfgDataxDI     : in  std_logic_vector(CNT_WIDTH-1 downto 0);
    -- control outputs
    donexSO        : out std_logic;
    -- data output
    samplexDO      : out std_logic_vector(31 downto 0);
    sampleValidxSO : out std_logic);
end sampleCounter;

architecture Behavioral of sampleCounter is

  signal counterxD        : std_logic_vector(CNT_WIDTH-1 downto 0);
  signal matchxS          : std_logic;
  signal donexSN, donexSP : std_logic := '0';

  signal cntTopxDN, cntTopxDP : std_logic_vector(CNT_WIDTH-1 downto 0);

  signal samplexDN, samplexDP           : std_logic_vector(31 downto 0);
  signal sampleValidxSN, sampleValidxSP : std_logic;

  -- dsp slice signals
  signal dspPxD   : std_logic_vector(47 downto 0);
  signal incCntxS : std_logic;
  signal clrCntxS : std_logic;
  
begin

  cntTopxDN <= cfgDataxDI when wrTopxSI = '1' else cntTopxDP;

  samplexDN      <= samplexDI;
  sampleValidxSN <= sampleValidxSI;
  samplexDO      <= samplexDP;
  sampleValidxSO <= sampleValidxSP;

  clrCntxS <= RstxRI;
  incCntxS <= runxSI and (not donexSP) and sampleValidxSI;

  matchxS <= '1' when counterxD = cntTopxDP else '0';
  donexSN <= (matchxS or donexSP) and runxSI;
  donexSO <= donexSP;

  regs : process (ClkxCI)
  begin  -- process regs
    if ClkxCI'event and ClkxCI = '1' then  -- rising clock edge
      samplexDP <= samplexDN;
      if RstxRI = '1' then
        donexSP        <= '0';
        sampleValidxSP <= '0';
        cntTopxDP      <= (others => '0');
      else
        donexSP        <= donexSN;
        sampleValidxSP <= sampleValidxSN;
        cntTopxDP      <= cntTopxDN;
      end if;
    end if;
  end process regs;

  -- DSP slice used as fast counter

  counterxD <= dspPxD(CNT_WIDTH-1 downto 0);

  DSP48A1_cntr_inst : DSP48A1
    generic map (
      A0REG       => 0,  -- First stage A input pipeline register (0/1)
      A1REG       => 1,  -- Second stage A input pipeline register (0/1)
      B0REG       => 0,  -- First stage B input pipeline register (0/1)
      B1REG       => 1,  -- Second stage B input pipeline register (0/1)
      CARRYINREG  => 1,  -- CARRYIN input pipeline register (0/1)
      CARRYINSEL  => "OPMODE5",  -- Specify carry-in source, "CARRYIN" or "OPMODE5" 
      CARRYOUTREG => 1,  -- CARRYOUT output pipeline register (0/1)
      CREG        => 1,                 -- C input pipeline register (0/1)
      DREG        => 1,  -- D pre-adder input pipeline register (0/1)
      MREG        => 1,                 -- M pipeline register (0/1)
      OPMODEREG   => 0,  -- Enable=1/disable=0 OPMODE input pipeline registers
      PREG        => 1,                 -- P output pipeline register (0/1)
      RSTTYPE     => "SYNC"     -- Specify reset type, "SYNC" or "ASYNC" 
      )
    port map (
      -- Cascade Ports: 18-bit (each) Ports to cascade from one DSP48 to another
      BCOUT     => open,                -- 18-bit B port cascade output
      PCOUT     => open,  -- 48-bit P cascade output (if used, connect to PCIN of another DSP48A1)
      -- Data Ports: 1-bit (each) Data input and output ports
      CARRYOUT  => open,  -- 1-bit carry output (if used, connect to CARRYIN pin of another DSP48A1)
      CARRYOUTF => open,                -- 1-bit fabric carry output
      M         => open,                -- 36-bit fabric multiplier data output
      P         => dspPxD,              -- 48-bit data output
      -- Cascade Ports: 48-bit (each) Ports to cascade from one DSP48 to another
      PCIN      => x"000000000000",  -- 48-bit P cascade input (if used, connect to PCOUT of another DSP48A1)
      -- Control Input Ports: 1-bit (each) Clocking and operation mode
      CLK       => ClkxCI,              -- 1-bit clock input
      OPMODE    => "00001110",  -- configure post adder to accumulate data on C port
      -- Data Ports: 18-bit (each) Data input and output ports
      A         => "000000000000000000",  -- 18-bit A data input
      B         => "000000000000000000",  -- 18-bit B data input (connected to fabric or BCOUT of adjacent DSP48A1)
      C         => x"000000000001",     -- 48-bit C data input
      CARRYIN   => '0',  -- 1-bit carry input signal (if used, connect to CARRYOUT pin of another
      -- DSP48A1)

      D          => "000000000000000000",  -- 18-bit B pre-adder data input
      -- Reset/Clock Enable Input Ports: 1-bit (each) Reset and enable input ports
      CEA        => '0',  -- 1-bit active high clock enable input for A registers
      CEB        => '0',  -- 1-bit active high clock enable input for B registers
      CEC        => '1',  -- 1-bit active high clock enable input for C
-- registers. We apply a fixed value to C so we simply enable this constantly
      CECARRYIN  => '0',  -- 1-bit active high clock enable input for CARRYIN registers
      CED        => '0',  -- 1-bit active high clock enable input for D registers
      CEM        => '0',  -- 1-bit active high clock enable input for multiplier registers
      CEOPMODE   => '1',  -- 1-bit active high clock enable input for OPMODE registers
      CEP        => incCntxS,  -- 1-bit active high clock enable input for P registers
      RSTA       => '0',  -- 1-bit reset input for A pipeline registers
      RSTB       => '0',  -- 1-bit reset input for B pipeline registers
      RSTC       => '0',  -- 1-bit reset input for C pipeline registers
      RSTCARRYIN => '0',  -- 1-bit reset input for CARRYIN pipeline registers
      RSTD       => '0',  -- 1-bit reset input for D pipeline registers
      RSTM       => '0',  -- 1-bit reset input for M pipeline registers
      RSTOPMODE  => '0',  -- 1-bit reset input for OPMODE pipeline registers
      RSTP       => clrCntxS   -- 1-bit reset input for P pipeline registers
      );


end Behavioral;
